/**
   @file
   @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BODYPLUGIN_BODY_MOTION_ITEM_H_INCLUDED
#define CNOID_BODYPLUGIN_BODY_MOTION_ITEM_H_INCLUDED

#include <cnoid/BodyMotion>
#include <cnoid/MultiSeqItem>
#include <cnoid/MultiValueSeqItem>
#include <cnoid/MultiSE3SeqItem>
#include <cnoid/Vector3SeqItem>
#include "exportdecl.h"

namespace cnoid {

    class CNOID_EXPORT BodyMotionItem : public AbstractMultiSeqItem
    {
      public:
        BodyMotionItem();

        virtual AbstractMultiSeqPtr abstractSeq() { return bodyMotion_; }

        inline const BodyMotionPtr& motion() { return bodyMotion_; }

        inline MultiValueSeqItem* jointPosSeqItem() {
            return jointPosSeqItem_.get();
        }

        inline const MultiValueSeqPtr& jointPosSeq() {
            return bodyMotion_->jointPosSeq();
        }

        inline MultiSE3SeqItem* linkPosSeqItem() {
            return linkPosSeqItem_.get();
        }
            
        inline const MultiSE3SeqPtr& linkPosSeq() {
            return bodyMotion_->linkPosSeq();
        }

        bool hasZMPseqItem() { return bodyMotion_->hasZMPseq(); }
        Vector3SeqItem* ZMPseqItem();

        inline Vector3SeqPtr ZMPseq() {
            return ZMPseqItem()->seq();
        }

        virtual void notifyUpdate();

        void updateChildItemLineup();

      protected:

        BodyMotionItem(const BodyMotionItem& org);

        virtual bool onNewChildItem(Item* childItem, bool isManualOperation);
        virtual ItemPtr doDuplicate() const;
        virtual bool store(Archive& archive);
        virtual bool restore(const Archive& archive);

      private:

        BodyMotionPtr bodyMotion_;
            
        MultiValueSeqItemPtr jointPosSeqItem_;
        MultiSE3SeqItemPtr linkPosSeqItem_;
        Vector3SeqItemPtr ZMPseqItem_;

        void initialize();
        void onSubItemUpdated(Item* childItem);
    };

    typedef boost::intrusive_ptr<BodyMotionItem> BodyMotionItemPtr;

    void initializeBodyMotionItem(ExtensionManager& ext);
        
}

#endif
