/**
   @file
   @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_UTIL_MULTI_SE3_SEQ_H_INCLUDED
#define CNOID_UTIL_MULTI_SE3_SEQ_H_INCLUDED

#include "MultiSeq.h"
#include "EigenTypes.h"
#include "exportdecl.h"

namespace cnoid {

    class Mapping;
    class Sequence;
    class YAMLWriter;
        
    class CNOID_EXPORT MultiSE3Seq : public MultiSeq<SE3, Eigen::aligned_allocator<SE3> >
	{
        typedef MultiSeq<SE3, Eigen::aligned_allocator<SE3> > BaseSeqType;

    public:
        typedef boost::shared_ptr<MultiSE3Seq> Ptr;
            
        MultiSE3Seq(int numParts = 1, int numFrames = 0, double frameRate = DEFAULT_FRAME_RATE);
        MultiSE3Seq(const MultiSE3Seq& org);
        virtual ~MultiSE3Seq();

        MultiSE3Seq& operator=(const MultiSE3Seq& rhs);
        
        //virtual bool loadPlainFormat(const std::string& filename);

        bool loadPlainMatrixFormat(const std::string& filename);
        bool saveTopPartAsPlainMatrixFormat(const std::string& filename);

    protected:
        virtual SE3 defaultValue() const { return SE3(Vector3::Zero(), Quat::Identity()); }

        virtual bool doWriteSeq(YAMLWriter& writer);
        virtual bool doReadSeq(const Mapping& archive);

    private:
        void readPosQuatSeq(int nParts, int nFrames, const Sequence& values, bool isWfirst);
        void readPosRpySeq(int nParts, int nFrames, const Sequence& values);
    };

    typedef MultiSE3Seq::Ptr MultiSE3SeqPtr;        
}

#endif
